#include <vector>
#include <algorithm>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>

int SDL_main(int, char**)
{
	//inicializace SDL - pouze video
	SDL_Init(SDL_INIT_VIDEO);
	IMG_Init(IMG_INIT_PNG);
	TTF_Init();
	//vytvoen okna
	SDL_Window* MainWindow = SDL_CreateWindow("Vykresleni textu", //jmno okna
		SDL_WINDOWPOS_CENTERED, //okno bude vytvoeno ve stedu obrazovky
		SDL_WINDOWPOS_CENTERED,
		800, 600, NULL); //ka, vka a flagy okna (ty nepotebujeme)


	//vytvoen rendereru
	SDL_Renderer* renderer = SDL_CreateRenderer(MainWindow, -1, SDL_RENDERER_TARGETTEXTURE);
	//oteven fontu
	TTF_Font* MyFont = TTF_OpenFontIndex("Fonts/PlayfairDisplay-Black.ttf", 140, 0);
	//vytvoen barev, se ktermi se bude pracovat
	SDL_Color* RedColor = new SDL_Color();
	RedColor->r = 255;
	RedColor->g = RedColor->b = 0;
	SDL_Color* GreenColor = new SDL_Color();
	GreenColor->g = 255;
	GreenColor->r = RedColor->b = 0;
	//vykreslen textu
	SDL_Surface* Solid = TTF_RenderText_Solid(MyFont, "Solid", *RedColor);
	SDL_Surface* Shaded = TTF_RenderText_Shaded(MyFont, "Shaded", *RedColor, *GreenColor);
	SDL_Surface* Blended = TTF_RenderText_Blended(MyFont, "Blended", *RedColor);
	//pevod SDL_Surface na SDL_Texturu, protoe grafick karta zpracovv koprovn rychleji
	SDL_Texture* SolidTexture = SDL_CreateTextureFromSurface(renderer, Solid);
	SDL_Texture* ShadedTexture = SDL_CreateTextureFromSurface(renderer, Shaded);
	SDL_Texture* BlendedTexture = SDL_CreateTextureFromSurface(renderer, Blended);
	//vytvoen vektoru pro seazen ky jednotlivch SDL_Surface
	std::vector<int> *VectorWithWidth=new std::vector<int>();
	VectorWithWidth->insert(VectorWithWidth->begin(), Solid->w);
	VectorWithWidth->insert(VectorWithWidth->begin(), Shaded->w);
	VectorWithWidth->insert(VectorWithWidth->begin(), Blended->w);
	std::sort(VectorWithWidth->begin(), VectorWithWidth->end());
	int* MaxWidth = new int;			//ziskn nejir SDL_Surface
	*MaxWidth=(*VectorWithWidth)[2];
	//vytvoen SDL_Texture, do kter se bude ve vykreslovat
	SDL_Texture* FinalTexture = SDL_CreateTexture(renderer, SDL_PIXELFORMAT_ARGB8888, 
		                        SDL_TEXTUREACCESS_TARGET, *MaxWidth, Solid->h*3);

	//obdelnk, kter udv, do kterch mst finln textury vykreslovat
	SDL_Rect* TargetRect = new SDL_Rect;

	SDL_SetRenderTarget(renderer,FinalTexture);					//zmna vykreslovn - renderer vykresluje do textury
	SDL_SetTextureBlendMode(FinalTexture, SDL_BLENDMODE_BLEND); //aktivace transparentn barvy
	SDL_SetRenderDrawColor(renderer,0,0,0,0);					//nastaven transparentn barvy
	SDL_RenderClear(renderer);									//vyplnn textury transparentn barvou jako pozad

	//vpoet, kam umstit text - aby byl vycentrovan
	//mus bt vypotno pesn, jinak bude vstup transformovan
	TargetRect->x = (*MaxWidth-Solid->w)/2;
	TargetRect->y = 0;
	TargetRect->w = Solid->w; TargetRect->h = Solid->h;
	//vykreslen "Solid" textu do textury
	SDL_RenderCopy(renderer,SolidTexture,NULL,TargetRect);

	//vpoet umstn textu
	TargetRect->x = (*MaxWidth - Shaded->w) / 2;
	TargetRect->y = TargetRect->h;
	TargetRect->w = Shaded->w; TargetRect->h = Shaded->h;
	//vykreslen "Shaded" textu do textury
	SDL_RenderCopy(renderer,ShadedTexture,NULL,TargetRect);

	//vpoet umstn poslednho textu
	TargetRect->x = (*MaxWidth - Blended->w) / 2;
	TargetRect->y = 2*TargetRect->h;
	TargetRect->w = Blended->w; TargetRect->h = Blended->h;
	//vykreslen "Blended" textu do textury
	SDL_RenderCopy(renderer,BlendedTexture,NULL,TargetRect);

	SDL_SetRenderTarget(renderer,NULL);	//renderer vykresluje zpt na obrazovku

	//klid
	delete MaxWidth;
	delete RedColor;
	delete GreenColor;
	delete VectorWithWidth;
	SDL_FreeSurface(Solid);
	SDL_FreeSurface(Shaded);
	SDL_FreeSurface(Blended);
	SDL_DestroyTexture(SolidTexture);
	SDL_DestroyTexture(ShadedTexture);
	SDL_DestroyTexture(BlendedTexture);

	//nastaven TargetRect tak, aby ml stejnou velikost jako textura
	//bude pouit pro vykreslen na obrazovku
	TargetRect->y = TargetRect->x = 0;
	TargetRect->h *= 3;





	bool End = false;								//promnn, kter d hlavn smyku
	SDL_Event* event = new SDL_Event;				//struktura, kter uchovv udlosti
	while (!End)									//zatek hlavn smyky
	{
		while (SDL_PollEvent(event))				//naten udlosti z fronty
		{
			if (event->type == SDL_QUIT)				//manipulace s udlost
				End = true;							//ukonen aplikace
			else if (event->type == SDL_KEYDOWN)		//reagovn na stisk klvesy
			{
				switch (event->key.keysym.sym)
				{
				case SDLK_RIGHT:					//ipka v pravo -> upraven x-ov pozice obdelnku
					TargetRect->x += 5; break;
				case SDLK_LEFT:						//ipka v levo -> upraven x-ov pozice obdelnku
					TargetRect->x -= 5; break;
				case SDLK_UP:						//ipka nahoru -> upraven y-ov pozice obdelnku
					TargetRect->y -= 5; break;
				case SDLK_DOWN:						//ipka dol -> upraven y-ov pozice obdelnku
					TargetRect->y += 5; break;
				}
			}
		}

		SDL_SetRenderDrawColor(renderer, 0, 0, 255, 255);		//nastaven barvy, kterou se bude vykreslovat pozad
		SDL_RenderClear(renderer);								//vykreslen pozad

		//vykreslen textu
		SDL_RenderCopy(renderer, FinalTexture, NULL, TargetRect);

		SDL_RenderPresent(renderer);



	}

	//nen poteba mazat jednotliv objekty, po ukonen programu voln msto automaticky promae OS

	TTF_Quit();								//ukonen SDL_ttf
	IMG_Quit();								//ukonen SDL_Image
	SDL_Quit();								//Ukonen SDL
	return 0;
}